@@ -1,4 +1,4 @@
-<?php // $Id: mod.php,v 1.80.2.3 2005/10/09 19:38:57 skodak Exp $
+<?php // $Id: mod.php,v 1.100.2.1 2006/05/15 17:52:39 stronk7 Exp $
 
 //  Moves, adds, updates, duplicates or deletes modules in a course
 
@@ -8,6 +8,21 @@
     require_login();
 
     $sectionreturn = optional_param('sr', '', PARAM_INT);
+    $add           = optional_param('add','', PARAM_ALPHA);
+    $type          = optional_param('type', '', PARAM_ALPHA);
+    $indent        = optional_param('indent', 0, PARAM_INT);
+    $update        = optional_param('update', 0, PARAM_INT);
+    $hide          = optional_param('hide', 0, PARAM_INT);
+    $show          = optional_param('show', 0, PARAM_INT);
+    $copy          = optional_param('copy', 0, PARAM_INT);
+    $moveto        = optional_param('moveto', 0, PARAM_INT);
+    $movetosection = optional_param('movetosection', 0, PARAM_INT);
+    $delete        = optional_param('delete', 0, PARAM_INT);
+    $course        = optional_param('course', 0, PARAM_INT);
+    $groupmode     = optional_param('groupmode', -1, PARAM_INT);
+    $duplicate     = optional_param('duplicate', 0, PARAM_INT);
+    $cancel        = optional_param('cancel', 0, PARAM_BOOL);
+    $cancelcopy    = optional_param('cancelcopy', 0, PARAM_BOOL);
 
     if (isset($SESSION->modform)) {   // Variables are stored in the session
         $mod = $SESSION->modform;
@@ -16,18 +31,18 @@
         $mod = (object)$_POST;
     }
 
-    if (isset($cancel)) {
+    if ($cancel) {
         if (!empty($SESSION->returnpage)) {
             $return = $SESSION->returnpage;
             unset($SESSION->returnpage);
             redirect($return);
         } else {
-            redirect("view.php?id=$mod->course#$sectionreturn");
+            redirect("view.php?id=$mod->course#section-$sectionreturn");
         }
     }
 
 
-    if (isset($_POST["course"]) and confirm_sesskey()) {    // add or update form submitted
+    if (!empty($course) and confirm_sesskey()) {    // add or update form submitted
 
         if (empty($mod->coursemodule)) { //add
             if (! $course = get_record("course", "id", $mod->course)) {
@@ -52,7 +67,7 @@
         }
         
         $mod->course = $course->id;
-        $mod->modulename = clean_filename($mod->modulename);  // For safety
+        $mod->modulename = clean_param($mod->modulename, PARAM_SAFEDIR);  // For safety
         $modlib = "$CFG->dirroot/mod/$mod->modulename/lib.php";
 
         if (file_exists($modlib)) {
@@ -112,7 +127,11 @@
 
             case "add":
 
-                if (trim($mod->name) == '') {
+                if (!course_allowed_module($course,$mod->modulename)) {
+                    error("This module ($mod->modulename) has been disabled for this particular course");
+                }
+
+                if (!isset($mod->name) || trim($mod->name) == '') {
                     $mod->name = get_string("modulename", $mod->modulename);
                 }
 
@@ -198,25 +217,25 @@
             unset($SESSION->returnpage);
             redirect($return);
         } else {
-            redirect("view.php?id=$course->id#$sectionreturn");
+            redirect("view.php?id=$course->id#section-$sectionreturn");
         }
         exit;
     }
 
-    if ((isset($_GET['movetosection']) or isset($_GET['moveto'])) and confirm_sesskey()) {
+    if ((!empty($movetosection) or !empty($moveto)) and confirm_sesskey()) {
 
         if (! $cm = get_record("course_modules", "id", $USER->activitycopy)) {
             error("The copied course module doesn't exist!");
         }
 
-        if (isset($_GET['movetosection'])) {
-            if (! $section = get_record("course_sections", "id", $_GET['movetosection'])) {
+        if (!empty($movetosection)) {
+            if (! $section = get_record("course_sections", "id", $movetosection)) {
                 error("This section doesn't exist");
             }
             $beforecm = NULL;
 
         } else {                      // normal moveto
-            if (! $beforecm = get_record("course_modules", "id", $_GET['moveto'])) {
+            if (! $beforecm = get_record("course_modules", "id", $moveto)) {
                 error("The destination course module doesn't exist");
             }
             if (! $section = get_record("course_sections", "id", $beforecm->section)) {
@@ -243,12 +262,12 @@
         if (SITEID == $section->course) {
             redirect($CFG->wwwroot);
         } else {
-            redirect("view.php?id=$section->course#$sectionreturn");
+            redirect("view.php?id=$section->course#section-$sectionreturn");
         }
 
-    } else if (isset($_GET['indent']) and confirm_sesskey()) {
+    } else if (!empty($indent) and confirm_sesskey()) {
 
-        require_variable($id);
+        $id = required_param('id',PARAM_INT);
 
         if (! $cm = get_record("course_modules", "id", $id)) {
             error("This course module doesn't exist");
@@ -258,7 +277,7 @@
             error("You can't modify this course!");
         }
 
-        $cm->indent += $_GET['indent'];
+        $cm->indent += $indent;
 
         if ($cm->indent < 0) {
             $cm->indent = 0;
@@ -271,13 +290,13 @@
         if (SITEID == $cm->course) {
             redirect($CFG->wwwroot);
         } else {
-            redirect("view.php?id=$cm->course#$sectionreturn");
+            redirect("view.php?id=$cm->course#section-$sectionreturn");
         }
         exit;
 
-    } else if (isset($_GET['hide']) and confirm_sesskey()) {
+    } else if (!empty($hide) and confirm_sesskey()) {
 
-        if (! $cm = get_record("course_modules", "id", $_GET['hide'])) {
+        if (! $cm = get_record("course_modules", "id", $hide)) {
             error("This course module doesn't exist");
         }
 
@@ -292,13 +311,13 @@
         if (SITEID == $cm->course) {
             redirect($CFG->wwwroot);
         } else {
-            redirect("view.php?id=$cm->course#$sectionreturn");
+            redirect("view.php?id=$cm->course#section-$sectionreturn");
         }
         exit;
 
-    } else if (isset($_GET['show']) and confirm_sesskey()) {
+    } else if (!empty($show) and confirm_sesskey()) {
 
-        if (! $cm = get_record("course_modules", "id", $_GET['show'])) {
+        if (! $cm = get_record("course_modules", "id", $show)) {
             error("This course module doesn't exist");
         }
 
@@ -322,13 +341,15 @@
         if (SITEID == $cm->course) {
             redirect($CFG->wwwroot);
         } else {
-            redirect("view.php?id=$cm->course#$sectionreturn");
+            redirect("view.php?id=$cm->course#section-$sectionreturn");
         }
         exit;
 
-    } else if (isset($_GET['groupmode']) and confirm_sesskey()) {
+    } else if ($groupmode > -1 and confirm_sesskey()) {
 
-        if (! $cm = get_record("course_modules", "id", $_GET['id'])) {
+        $id = required_param( 'id', PARAM_INT );
+
+        if (! $cm = get_record("course_modules", "id", $id)) {
             error("This course module doesn't exist");
         }
 
@@ -336,20 +357,20 @@
             error("You can't modify this course!");
         }
 
-        set_coursemodule_groupmode($cm->id, $_GET['groupmode']);
+        set_coursemodule_groupmode($cm->id, $groupmode);
 
         rebuild_course_cache($cm->course);
 
         if (SITEID == $cm->course) {
             redirect($CFG->wwwroot);
         } else {
-            redirect("view.php?id=$cm->course#$sectionreturn");
+            redirect("view.php?id=$cm->course#section-$sectionreturn");
         }
         exit;
 
-    } else if (isset($_GET['copy']) and confirm_sesskey()) { // value = course module
+    } else if (!empty($copy) and confirm_sesskey()) { // value = course module
 
-        if (! $cm = get_record("course_modules", "id", $_GET['copy'])) {
+        if (! $cm = get_record("course_modules", "id", $copy)) {
             error("This course module doesn't exist");
         }
 
@@ -373,9 +394,9 @@
         $USER->activitycopycourse = $cm->course;
         $USER->activitycopyname = $instance->name;
 
-        redirect("view.php?id=$cm->course#$sectionreturn");
+        redirect("view.php?id=$cm->course#section-$sectionreturn");
 
-    } else if (isset($_GET['cancelcopy']) and confirm_sesskey()) { // value = course module
+    } else if (!empty($cancelcopy) and confirm_sesskey()) { // value = course module
 
         $courseid = $USER->activitycopycourse;
 
@@ -383,11 +404,11 @@
         unset($USER->activitycopycourse);
         unset($USER->activitycopyname);
 
-        redirect("view.php?id=$courseid#$sectionreturn");
+        redirect("view.php?id=$courseid#section-$sectionreturn");
 
-    } else if (isset($_GET['delete']) and confirm_sesskey()) {   // value = course module
+    } else if (!empty($delete) and confirm_sesskey()) {   // value = course module
 
-        if (! $cm = get_record("course_modules", "id", $_GET['delete'])) {
+        if (! $cm = get_record("course_modules", "id", $delete)) {
             error("This course module doesn't exist");
         }
 
@@ -426,24 +447,25 @@
         $form->instancename = $instance->name;
         $form->sesskey      = !empty($USER->id) ? $USER->sesskey : '';
 
-        $strdeletecheck = get_string("deletecheck", "", "$form->fullmodulename");
-        $strdeletecheckfull = get_string("deletecheckfull", "", "$form->fullmodulename '$form->instancename'");
+        $strdeletecheck = get_string('deletecheck', '', $form->fullmodulename);
+        $strdeletecheckfull = get_string('deletecheckfull', '', "$form->fullmodulename '$form->instancename'");
 
-        print_header_simple("$strdeletecheck", "",
-                     "$strdeletecheck");
+        $CFG->pagepath = 'mod/'.$module->name.'/delete';
 
-        print_simple_box_start("center", "60%", "#FFAAAA", 20, "noticebox");
+        print_header_simple($strdeletecheck, '', $strdeletecheck);
+
+        print_simple_box_start('center', '60%', '#FFAAAA', 20, 'noticebox');
         print_heading($strdeletecheckfull);
-        include_once("mod_delete.html");
+        include_once('mod_delete.html');
         print_simple_box_end();
         print_footer($course);
 
         exit;
 
 
-    } else if (isset($_GET['update']) and confirm_sesskey()) {   // value = course module
+    } else if (!empty($update) and confirm_sesskey()) {   // value = course module
 
-        if (! $cm = get_record("course_modules", "id", $_GET['update'])) {
+        if (! $cm = get_record("course_modules", "id", $update)) {
             error("This course module doesn't exist");
         }
 
@@ -483,7 +505,7 @@
         $sectionname    = get_string("name$course->format");
         $fullmodulename = strtolower(get_string("modulename", $module->name));
 
-        if ($form->section) {
+        if ($form->section && $course->format != 'site') {
             $heading->what = $fullmodulename;
             $heading->in   = "$sectionname $cw->section";
             $pageheading = get_string("updatingain", "moodle", $heading);
@@ -491,10 +513,17 @@
             $pageheading = get_string("updatinga", "moodle", $fullmodulename);
         }
         $strnav = "<a href=\"$CFG->wwwroot/mod/$module->name/view.php?id=$cm->id\">".format_string($form->name,true)."</a> ->";
+        
+        if ($module->name == 'resource') {
+            $CFG->pagepath = 'mod/'.$module->name.'/'.$form->type;
+        } else {
+            $CFG->pagepath = 'mod/'.$module->name.'/mod';
+        }
 
-    } else if (isset($_GET['duplicate']) and confirm_sesskey()) {   // value = course module
+    } else if (!empty($duplicate) and confirm_sesskey()) {   // value = course module
 
-        if (! $cm = get_record("course_modules", "id", $_GET['duplicate'])) {
+
+        if (! $cm = get_record("course_modules", "id", $duplicate)) {
             error("This course module doesn't exist");
         }
 
@@ -545,25 +574,26 @@
         }
         $strnav = "<a href=\"$CFG->wwwroot/mod/$module->name/view.php?id=$cm->id\">$form->name</a> ->";
 
+        $CFG->pagepath = 'mod/'.$module->name.'/mod';
 
-    } else if (isset($_GET['add']) and confirm_sesskey()) {
 
-        if (empty($_GET['add'])) {
-            redirect($_SERVER["HTTP_REFERER"]);
-            die;
-        }
+    } else if (!empty($add) and confirm_sesskey()) {
 
-        require_variable($id);
-        require_variable($section);
+        $id = required_param('id',PARAM_INT);
+        $section = required_param('section',PARAM_INT);
 
         if (! $course = get_record("course", "id", $id)) {
             error("This course doesn't exist");
         }
 
-        if (! $module = get_record("modules", "name", $_GET['add'])) {
+        if (! $module = get_record("modules", "name", $add)) {
             error("This module type doesn't exist");
         }
 
+        if (!course_allowed_module($course,$module->id)) {
+            error("This module has been disabled for this particular course");
+        }
+
         $form->section    = $section;         // The section number itself
         $form->course     = $course->id;
         $form->module     = $module->id;
@@ -572,14 +602,14 @@
         $form->coursemodule = "";
         $form->mode       = "add";
         $form->sesskey    = !empty($USER->id) ? $USER->sesskey : '';
-        if (isset($_GET['type'])) {
-            $form->type = $_GET['type'];
+        if (!empty($type)) {
+            $form->type = $type;
         }
 
         $sectionname    = get_string("name$course->format");
         $fullmodulename = get_string("modulename", $module->name);
 
-        if ($form->section) {
+        if ($form->section && $course->format != 'site') {
             $heading->what = $fullmodulename;
             $heading->to   = "$sectionname $form->section";
             $pageheading = get_string("addinganewto", "moodle", $heading);
@@ -588,6 +618,14 @@
         }
         $strnav = '';
 
+        $CFG->pagepath = 'mod/'.$module->name;
+        if (!empty($type)) {
+            $CFG->pagepath .= '/' . $type;
+        }
+        else {
+            $CFG->pagepath .= '/mod';
+        }
+
     } else {
         error("No action was specfied");
     }
@@ -622,7 +660,7 @@
             $defaultformat = FORMAT_MOODLE;
         }
 
-        $icon = "<img align=\"middle\" height=\"16\" width=\"16\" src=\"$CFG->modpixpath/$module->name/icon.gif\" alt=\"\" />&nbsp;";
+        $icon = '<img align="middle" height="16" width="16" src="'.$CFG->modpixpath.'/'.$module->name.'/icon.gif" alt="" style="vertical-align: middle;" />&nbsp;';
 
         print_heading_with_help($pageheading, "mods", $module->name, $icon);
         print_simple_box_start('center', '', '', 5, 'generalbox', $module->name);
@@ -638,5 +676,4 @@
     }
 
     print_footer($course);
-
 ?>
