@@ -1,4 +1,4 @@
-<?php //$Id: restorelib.php,v 1.30.2.4 2005/08/03 19:49:19 stronk7 Exp $
+<?php //$Id: restorelib.php,v 1.42.2.3 2006/06/06 17:23:42 tjhunt Exp $
     //This php script contains all the stuff to backup/restore
     //forum mods
 
@@ -65,6 +65,9 @@
             $forum->rsstype = backup_todb($info['MOD']['#']['RSSTYPE']['0']['#']);
             $forum->rssarticles = backup_todb($info['MOD']['#']['RSSARTICLES']['0']['#']);
             $forum->timemodified = backup_todb($info['MOD']['#']['TIMEMODIFIED']['0']['#']);
+            $forum->warnafter = backup_todb($info['MOD']['#']['WARNAFTER']['0']['#']);
+            $forum->blockafter = backup_todb($info['MOD']['#']['BLOCKAFTER']['0']['#']);
+            $forum->blockperiod = backup_todb($info['MOD']['#']['BLOCKPERIOD']['0']['#']);
 
             //We have to recode the scale field if it's <0 (positive is a grade, not a scale)
             if ($forum->scale < 0) {
@@ -92,7 +95,9 @@
             }
 
             //Do some output
-            echo "<li>".get_string("modulename","forum")." \"".format_string(stripslashes($forum->name),true)."\"";
+            if (!defined('RESTORE_SILENTLY')) {
+                echo "<li>".get_string("modulename","forum")." \"".format_string(stripslashes($forum->name),true)."\"";
+            }
             backup_flush(300);
 
             if ($newid) {
@@ -100,7 +105,7 @@
                 backup_putid($restore->backup_unique_code,$mod->modtype,
                              $mod->id, $newid);
                 //Now check if want to restore user data and do it.
-                if ($restore->mods['forum']->userinfo) {
+                if (restore_userdata_selected($restore,'forum',$mod->id)) {
                     //Restore forum_subscriptions
                     $status = forum_subscriptions_restore_mods ($newid,$info,$restore);
                     if ($status) {
@@ -112,6 +117,36 @@
                         $status = forum_read_restore_mods ($newid,$info,$restore);
                     }
                 }
+
+                // If forum type is single, just recreate the initial discussion/post automatically
+                // if it hasn't been created still (because no user data was selected on backup or
+                // restore.
+                if ($forum->type == 'single' && !record_exists('forum_discussions', 'forum', $newid)) {
+                    //Load forum/lib.php
+                    require_once ($CFG->dirroot.'/mod/forum/lib.php');
+                    // Calculate the default format
+                    if (can_use_html_editor()) {
+                        $defaultformat = FORMAT_HTML;
+                    } else {
+                        $defaultformat = FORMAT_MOODLE;
+                    }
+                    //Create discussion/post data
+                    $sd = new stdClass;
+                    $sd->course   = $forum->course;
+                    $sd->forum    = $newid;
+                    $sd->name     = $forum->name;
+                    $sd->intro    = $forum->intro;
+                    $sd->assessed = $forum->assessed;
+                    $sd->format   = $defaultformat;
+                    $sd->mailnow  = false;
+                    //Insert dicussion/post data
+                    $sdid = forum_add_discussion($sd, $sd->intro);
+                    //Now, mark the initial post of the discussion as mailed!
+                    if ($sdid) {
+                        set_field ('forum_posts','mailed', '1', 'discussion', $sdid);
+                    }
+                }
+
             } else {
                 $status = false;
             }
@@ -130,7 +165,11 @@
         $status = true;
 
         //Get the discussions array
-        $subscriptions = $info['MOD']['#']['SUBSCRIPTIONS']['0']['#']['SUBSCRIPTION'];
+        if (!empty($info['MOD']['#']['SUBSCRIPTIONS'])) {
+            $subscriptions = $info['MOD']['#']['SUBSCRIPTIONS']['0']['#']['SUBSCRIPTION'];
+        } else {
+            $subscriptions = array();
+        }
 
         //Iterate over subscriptions
         for($i = 0; $i < sizeof($subscriptions); $i++) {
@@ -158,9 +197,11 @@
 
             //Do some output
             if (($i+1) % 50 == 0) {
-                echo ".";
-                if (($i+1) % 1000 == 0) {
-                    echo "<br />";
+                if (!defined('RESTORE_SILENTLY')) {
+                    echo ".";
+                    if (($i+1) % 1000 == 0) {
+                        echo "<br />";
+                    }
                 }
                 backup_flush(300);
             }
@@ -185,7 +226,11 @@
         $status = true;
 
         //Get the discussions array
-        $discussions = $info['MOD']['#']['DISCUSSIONS']['0']['#']['DISCUSSION'];
+        if (!empty($info['MOD']['#']['DISCUSSIONS'])) {
+            $discussions = $info['MOD']['#']['DISCUSSIONS']['0']['#']['DISCUSSION'];
+        } else {
+            $discussions = array();
+        }
 
         //Iterate over discussions
         for($i = 0; $i < sizeof($discussions); $i++) {
@@ -208,6 +253,8 @@
             $discussion->assessed = backup_todb($dis_info['#']['ASSESSED']['0']['#']);
             $discussion->timemodified = backup_todb($dis_info['#']['TIMEMODIFIED']['0']['#']);
             $discussion->usermodified = backup_todb($dis_info['#']['USERMODIFIED']['0']['#']);
+            $discussion->timestart = backup_todb($dis_info['#']['TIMESTART']['0']['#']);
+            $discussion->timeend = backup_todb($dis_info['#']['TIMEEND']['0']['#']);
 
             //We have to recode the userid field
             $user = backup_getid($restore->backup_unique_code,"user",$discussion->userid);
@@ -232,9 +279,11 @@
 
             //Do some output
             if (($i+1) % 50 == 0) {
-                echo ".";
-                if (($i+1) % 1000 == 0) {
-                    echo "<br />";
+                if (!defined('RESTORE_SILENTLY')) {
+                    echo ".";
+                    if (($i+1) % 1000 == 0) {
+                        echo "<br />";
+                    }
                 }
                 backup_flush(300);
             }
@@ -282,7 +331,11 @@
         $status = true;
 
         //Get the read array
-        $readposts = $info['MOD']['#']['READPOSTS']['0']['#']['READ'];
+        if (!empty($info['MOD']['#']['READPOSTS'])) {
+            $readposts = $info['MOD']['#']['READPOSTS']['0']['#']['READ'];
+        } else {
+            $readposts = array();
+        }
 
         //Iterate over readposts
         for($i = 0; $i < sizeof($readposts); $i++) {
@@ -337,9 +390,11 @@
 
             //Do some output
             if (($i+1) % 50 == 0) {
-                echo ".";
-                if (($i+1) % 1000 == 0) {
-                    echo "<br />";
+                if (!defined('RESTORE_SILENTLY')) {
+                    echo ".";
+                    if (($i+1) % 1000 == 0) {
+                        echo "<br />";
+                    }
                 }
                 backup_flush(300);
             }
@@ -389,6 +444,7 @@
             $post->format = backup_todb($pos_info['#']['FORMAT']['0']['#']);
             $post->attachment = backup_todb($pos_info['#']['ATTACHMENT']['0']['#']);
             $post->totalscore = backup_todb($pos_info['#']['TOTALSCORE']['0']['#']);
+            $post->mailnow = backup_todb($pos_info['#']['MAILNOW']['0']['#']);
 
             //We have to recode the userid field
             $user = backup_getid($restore->backup_unique_code,"user",$post->userid);
@@ -401,9 +457,11 @@
 
             //Do some output
             if (($i+1) % 50 == 0) {
-                echo ".";
-                if (($i+1) % 1000 == 0) {
-                    echo "<br />";
+                if (!defined('RESTORE_SILENTLY')) {
+                    echo ".";
+                    if (($i+1) % 1000 == 0) {
+                        echo "<br />";
+                    }
                 }
                 backup_flush(300);
             }
@@ -548,9 +606,11 @@
 
             //Do some output
             if (($i+1) % 50 == 0) {
-                echo ".";
-                if (($i+1) % 1000 == 0) {
-                    echo "<br />";
+                if (!defined('RESTORE_SILENTLY')) {
+                    echo ".";
+                    if (($i+1) % 1000 == 0) {
+                        echo "<br />";
+                    }
                 }
                 backup_flush(300);
             }
@@ -599,9 +659,11 @@
                 //Do some output
                 $i++;
                 if (($i+1) % 1 == 0) {
-                    echo ".";
-                    if (($i+1) % 20 == 0) {
-                        echo "<br />";
+                    if (!defined('RESTORE_SILENTLY')) {
+                        echo ".";
+                        if (($i+1) % 20 == 0) {
+                            echo "<br />";
+                        }
                     }
                     backup_flush(300);
                 }
@@ -763,7 +825,7 @@
                     //Get the post record from database
                     $dbpos = get_record("forum_posts","id","$pos->new_id");
                     if ($dbpos) {
-                        $log->url = "discuss.php?d=".$dbpos->discussion."&amp;parent=".$pos->new_id;
+                        $log->url = "discuss.php?d=".$dbpos->discussion."&parent=".$pos->new_id;
                         $log->info = $pos->new_id;
                         $status = true;
                     }
@@ -793,7 +855,7 @@
                     //Get the post record from database
                     $dbpos = get_record("forum_posts","id","$pos->new_id");
                     if ($dbpos) {
-                        $log->url = "discuss.php?d=".$dbpos->discussion."&amp;parent=".$pos->new_id;
+                        $log->url = "discuss.php?d=".$dbpos->discussion."&parent=".$pos->new_id;
                         $log->info = $pos->new_id;
                         $status = true;
                     }
@@ -813,11 +875,24 @@
             }
             break;
         case "search":
-            $log->url = "search.php?id=".$log->course."&amp;search=".urlencode($log->info);
+            $log->url = "search.php?id=".$log->course."&search=".urlencode($log->info);
             $status = true;
             break;
+        case "user report":
+            //recode the info field (it's the user id)
+            $user = backup_getid($restore->backup_unique_code,"user",$log->info);
+            if ($user) {
+                $log->info = $user->new_id;
+                //Now, extract the mode from the url field
+                $mode = substr(strrchr($log->url,"="),1);
+                $log->url = "user.php?course=".$log->course."&id=".$log->info."&mode=".$mode;
+                $status = true;
+            }
+            break;
         default:
-            echo "action (".$log->module."-".$log->action.") unknow. Not restored<br />";                 //Debug
+            if (!defined('RESTORE_SILENTLY')) {
+                echo "action (".$log->module."-".$log->action.") unknown. Not restored<br />";                 //Debug
+            }
             break;
         }
 
@@ -1021,14 +1096,18 @@
                     $post->message = addslashes($result);
                     $status = update_record("forum_posts",$post);
                     if ($CFG->debug>7) {
-                        echo '<br /><hr />'.htmlentities($content).'<br />changed to<br />'.htmlentities($result).'<hr /><br />';
+                        if (!defined('RESTORE_SILENTLY')) {
+                            echo '<br /><hr />'.s($content).'<br />changed to<br />'.s($result).'<hr /><br />';
+                        }
                     }
                 }
                 //Do some output
                 if (($i+1) % 5 == 0) {
-                    echo ".";
-                    if (($i+1) % 100 == 0) {
-                        echo "<br />";
+                    if (!defined('RESTORE_SILENTLY')) {
+                        echo ".";
+                        if (($i+1) % 100 == 0) {
+                            echo "<br />";
+                        }
                     }
                     backup_flush(300);
                 }
@@ -1051,14 +1130,18 @@
                     $forum->intro = addslashes($result);
                     $status = update_record("forum",$forum);
                     if ($CFG->debug>7) {
-                        echo '<br /><hr />'.htmlentities($content).'<br />changed to<br />'.htmlentities($result).'<hr /><br />';
+                        if (!defined('RESTORE_SILENTLY')) {
+                            echo '<br /><hr />'.s($content).'<br />changed to<br />'.s($result).'<hr /><br />';
+                        }
                     }
                 }
                 //Do some output
                 if (($i+1) % 5 == 0) {
-                    echo ".";
-                    if (($i+1) % 100 == 0) {
-                        echo "<br />";
+                    if (!defined('RESTORE_SILENTLY')) {
+                        echo ".";
+                        if (($i+1) % 100 == 0) {
+                            echo "<br />";
+                        }
                     }
                     backup_flush(300);
                     }
