@@ -1,40 +1,45 @@
-<?php
+<?php // $Id: continue.php,v 1.19.2.3 2006/05/25 08:12:30 skodak Exp $
+
 /****************** continue ************************************/
 
     confirm_sesskey();
 
     // left menu code
+    // check to see if the user can see the left menu
+    if (!isteacher($course->id)) {
+        $lesson->displayleft = lesson_displayleftif($lesson);
+    }
     if ($lesson->displayleft) {
        if($firstpageid = get_field('lesson_pages', 'id', 'lessonid', $lesson->id, 'prevpageid', 0)) {
-               echo '<table><tr valign="top"><td>';
             // print the pages
-            echo '<form name="lessonpages2" method="post" action="view.php">';
-            echo '<input type="hidden" name="id" value="'. $cm->id .'" />';
-            echo '<input type="hidden" name="action" value="navigation" />';
-            echo '<input type="hidden" name="pageid" />';
-                    echo "<table><tr><td valign=\"top\">";
-                    echo "<div class=\"leftmenutable\">".get_string('lessonmenu', 'lesson')."<br />";
-                    echo "<div class=\"main\">";
-                    echo "<a href=\"../../course/view.php?id=$course->id\">".get_string("mainmenu", "lesson")."</a><br />";                                    echo "</div>";
-                    echo '<div class="leftmenu">';
-                    lesson_print_tree_menu($lesson->id, $firstpageid, $cm->id);
-                    echo '</div></div></td></tr></table>'; //close lmlinks
-            echo '</form>';
+            echo '<table><tr valign="top"><td>';
+            // skip navigation link
+            echo '<a href="#maincontent" class="skip">'.get_string('skip', 'lesson').'</a>';
+            echo "<div class=\"leftmenu_container\">\n";
+                echo '<div class="leftmenu_title">'.get_string('lessonmenu', 'lesson')."</div>\n";
+                echo "<div class=\"leftmenu_courselink\">\n";
+                echo "<a href=\"$CFG->wwwroot/course/view.php?id=$course->id\">".get_string("mainmenu", "lesson")."</a>\n";
+                echo "</div>\n";
+                echo "<div class=\"leftmenu_links\">\n";
+                lesson_print_tree_menu($lesson->id, $firstpageid, $cm->id);
+                echo "</div>\n";
+            echo "</div>\n";
             echo '</td><td align="center" width="100%">';
+            // skip to anchor
+            echo '<a name="maincontent" id="maincontent" title="'.get_string('anchortitle', 'lesson').'"></a>';
         }
     }
 
-    /// CDC-FLAG /// 6/21/04  This is the warning msg for teachers to inform them that cluster and unseen does not work while logged in as a teacher
+    // This is the warning msg for teachers to inform them that cluster and unseen does not work while logged in as a teacher
     if(isteacher($course->id)) {
-        if (execute_teacherwarning($lesson->id)) {
+        if (lesson_display_teacher_warning($lesson->id)) {
             $warningvars->cluster = get_string("clusterjump", "lesson");
             $warningvars->unseen = get_string("unseenpageinbranch", "lesson");
             echo "<p align=\"center\">".get_string("teacherjumpwarning", "lesson", $warningvars)."</p>";
         }
     }        
-    /// CDC-FLAG ///
 
-    /// CDC-FLAG /// 6/14/04 -- This is the code updates the lesson time for a timed test
+    // This is the code updates the lesson time for a timed test
     // get time information for this user
     if (!isteacher($course->id)) {
         if (!$timer = get_records_select('lesson_timer', "lessonid = $lesson->id AND userid = $USER->id", 'starttime')) {
@@ -82,24 +87,20 @@
             error("Error: could not update lesson_timer table");
         }
     }
-    /// CDC-FLAG ///            
 
     // record answer (if necessary) and show response (if none say if answer is correct or not)
-    if (empty($_POST['pageid'])) {
-        error("Continue: pageid missing");
-    }
     $pageid = required_param('pageid', PARAM_INT);
     if (!$page = get_record("lesson_pages", "id", $pageid)) {
         error("Continue: Page record not found");
     }
     // set up some defaults
-    $answerid = 0;
-    $noanswer = false;
-    $correctanswer = false;
-    $isessayquestion = false; // use this to turn off review button on essay questions
-    $newpageid = 0; // stay on the page
+    $answerid        = 0;
+    $noanswer        = false;
+    $correctanswer   = false;
+    $isessayquestion = false;   // use this to turn off review button on essay questions
+    $newpageid       = 0;       // stay on the page
+    $studentanswer   = '';      // use this to store student's answer(s) in order to display it on feedback page
     switch ($page->qtype) {
-        /// CDC-FLAG ///
          case LESSON_ESSAY :
             $isessayquestion = true;
             if (!$useranswer = $_POST['answer']) {
@@ -112,7 +113,7 @@
                 error("Continue: No answers found");
             }
             $correctanswer = false;
-            $response = "Your essay will be graded by the course instructor.";
+            $response = get_string('defaultessayresponse', 'lesson');
             foreach ($answers as $answer) {
                 $answerid = $answer->id;
                 $newpageid = $answer->jumpto;
@@ -124,9 +125,9 @@
             $userresponse->answer = $useranswer;
             $userresponse->response = "";
             $userresponse = addslashes(serialize($userresponse));
-        
-             break;
-        /// CDC-FLAG ///
+            
+            $studentanswer = $useranswer;
+            break;
          case LESSON_SHORTANSWER :
             if (!$useranswer = $_POST['answer']) {
                 $noanswer = true;
@@ -134,66 +135,108 @@
             }            
             $useranswer = stripslashes(clean_param($useranswer, PARAM_CLEAN));
             $userresponse = addslashes($useranswer);
-
             if (!$answers = get_records("lesson_answers", "pageid", $pageid, "id")) {
                 error("Continue: No answers found");
             }
-
+            $i=0;
             foreach ($answers as $answer) {
-                // massage the wild cards (if present)
-                if (strpos(' '.$answer->answer, '*')) {
-                    $answer->answer = str_replace('\*','@@@@@@', $answer->answer);
-                    $answer->answer = str_replace('*','.*', $answer->answer);
-                    $answer->answer = str_replace('@@@@@@', '\*', $answer->answer);    
-                }
-                $answer->answer = str_replace('.*','@@@@@@', $answer->answer);
-                $answer->answer = preg_quote($answer->answer, '/');
-                $answer->answer = str_replace('@@@@@@', '.*', $answer->answer);    
+                $i += 1;
+                $expectedanswer  = $answer->answer; // for easier handling of $answer->answer
+                $ismatch         = false; 
+                $markit          = false; 
+                $useregexp       = false;
 
-                if (lesson_iscorrect($pageid, $answer->jumpto) or 
-                    ($lesson->custom && $answer->score > 0) ) {
-                    if ($page->qoption) {
-                        // case sensitive
-                        if (preg_match('/^'.$answer->answer.'$/', $useranswer)) {
-                            $correctanswer = true;
-                            $answerid = $answer->id;
-                            $newpageid = $answer->jumpto;
-                            if (trim(strip_tags($answer->response))) {
-                                $response = $answer->response;
-                            }
-                            break;
-                        }
-                    } else {
-                        // case insensitive
-                        if (preg_match('/^'.$answer->answer.'$/i', $useranswer)) {
-                            $correctanswer = true;
-                            $answerid = $answer->id;
-                            $newpageid = $answer->jumpto;
-                            if (trim(strip_tags($answer->response))) {
-                                $response = $answer->response;
-                            }
-                            break;
-                        }
+                if ($page->qoption) {
+                    $useregexp = true;
+                }
+                
+                if ($useregexp) { //we are using 'normal analysis', which ignores case
+                    $ignorecase = '';
+                    if ( substr($expectedanswer,strlen($expectedanswer) - 2, 2) == '/i') {
+                        $expectedanswer = substr($expectedanswer,0,strlen($expectedanswer) - 2);
+                        $ignorecase = 'i';
                     }
                 } else {
-                    // see if user typed in any of the wrong answers
-                    // don't worry about case
-                    if (preg_match('/^'.$answer->answer.'$/i', $useranswer)) {
-                        $newpageid = $answer->jumpto;
-                        $answerid = $answer->id;
-                        if (trim(strip_tags($answer->response))) {
-                            $response = $answer->response;
+                    $expectedanswer = str_replace('*', '#####', $expectedanswer);
+                    $expectedanswer = preg_quote($expectedanswer, '/');
+                    $expectedanswer = str_replace('#####', '.*', $expectedanswer);
+                }
+                // see if user typed in any of the correct answers
+                if (lesson_iscorrect($pageid, $answer->jumpto) or ($lesson->custom && $answer->score > 0) ) {
+                    if (!$useregexp) { // we are using 'normal analysis', which ignores case
+                        if (preg_match('/^'.$expectedanswer.'$/i',$useranswer)) {
+                            $ismatch = true;
+                        }
+                    } else {
+                        if (preg_match('/^'.$expectedanswer.'$/'.$ignorecase,$useranswer)) {
+                            $ismatch = true;
                         }
                     }
+                    if ($ismatch == true) {
+                        $correctanswer = true;
+                    }
+                } else {
+                   if (!$useregexp) { //we are using 'normal analysis'
+                        // see if user typed in any of the wrong answers; don't worry about case
+                        if (preg_match('/^'.$expectedanswer.'$/i',$useranswer)) {
+                            $ismatch = true;
+                        }
+                    } else { // we are using regular expressions analysis
+                        $startcode = substr($expectedanswer,0,2);
+                        switch ($startcode){
+                            //1- check for absence of required string in $useranswer (coded by initial '--')
+                            case "--":
+                                $expectedanswer = substr($expectedanswer,2);
+                                if (!preg_match('/^'.$expectedanswer.'$/'.$ignorecase,$useranswer)) {
+                                    $ismatch = true;
+                                }
+                                break;                                      
+                            //2- check for code for marking wrong strings (coded by initial '++')
+                            case "++":
+                                $expectedanswer=substr($expectedanswer,2);
+                                $markit = true;
+                                //check for one or several matches
+                                if (preg_match_all('/'.$expectedanswer.'/'.$ignorecase,$useranswer, $matches)) {
+                                    $ismatch   = true;
+                                    $nb        = count($matches[0]);
+                                    $original  = array(); 
+                                    $marked    = array();
+                                    $fontStart = '<span class="incorrect matches">';
+                                    $fontEnd   = '</span>';
+                                    for ($i = 0; $i < $nb; $i++) {
+                                        array_push($original,$matches[0][$i]);
+                                        array_push($marked,$fontStart.$matches[0][$i].$fontEnd);
+                                    }
+                                    $useranswer = str_replace($original, $marked, $useranswer);
+                                }
+                                break;
+                            //3- check for wrong answers belonging neither to -- nor to ++ categories 
+                            default:
+                                if (preg_match('/^'.$expectedanswer.'$/'.$ignorecase,$useranswer, $matches)) {
+                                    $ismatch = true;
+                                }
+                                break;
+                        }
+                        $correctanswer = false;
+                    }
+                }
+                if ($ismatch) {
+                    $newpageid = $answer->jumpto;
+                    if (trim(strip_tags($answer->response))) {
+                        $response = $answer->response;
+                    }
+                    $answerid = $answer->id;
+                    break; // quit answer analysis immediately after a match has been found
                 }
             }
-            if (!isset($response)) {
+            if (!isset($response)) { //if no feedback message provided, use default message
                 if ($correctanswer) {
                     $response = get_string("thatsthecorrectanswer", "lesson");
                 } else {
                     $response = get_string("thatsthewronganswer", "lesson");
                 }
             }
+            $studentanswer = $useranswer;
             break;
         
         case LESSON_TRUEFALSE :
@@ -208,7 +251,6 @@
             if (lesson_iscorrect($pageid, $answer->jumpto)) {
                 $correctanswer = true;
             }
-            /* CDC-FLAG */  
             if ($lesson->custom) {
                 if ($answer->score > 0) {
                     $correctanswer = true;
@@ -216,7 +258,6 @@
                     $correctanswer = false;
                 }
             }
-            /// CDC-FLAG 6/21/04 ///
             $newpageid = $answer->jumpto;
             if (!$response = trim($answer->response)) {
                 if ($correctanswer) {
@@ -225,6 +266,7 @@
                     $response = get_string("thatsthewronganswer", "lesson");
                 }
             }
+            $studentanswer = $answer->answer;
             break;
         
         case LESSON_MULTICHOICE :
@@ -249,7 +291,15 @@
                 $nhits = 0;
                 $correctresponse = '';
                 $wrongresponse = '';
-                /// CDC-FLAG /// 6/11/04 this is for custom scores.  If score on answer is positive, it is correct                    
+                // store student's answers for displaying on feedback page
+                foreach ($answers as $answer) {
+                    foreach ($useranswers as $key => $answerid) {
+                        if ($answerid == $answer->id) {
+                            $studentanswer .= '<br />'.$answer->answer;
+                        }
+                    }
+                }
+                // this is for custom scores.  If score on answer is positive, it is correct                    
                 if ($lesson->custom) {
                     $ncorrect = 0;
                     $nhits = 0;
@@ -314,7 +364,6 @@
                         }
                     }
                 }
-                /// CDC-FLAG ///
                 if ((count($useranswers) == $ncorrect) and ($nhits == $ncorrect)) {
                     $correctanswer = true;
                     if (!$response = $correctresponse) {
@@ -340,7 +389,6 @@
                 if (lesson_iscorrect($pageid, $answer->jumpto)) {
                     $correctanswer = true;
                 }
-                /* CDC-FLAG */
                 if ($lesson->custom) {
                     if ($answer->score > 0) {
                         $correctanswer = true;
@@ -348,7 +396,6 @@
                         $correctanswer = false;
                     }
                 }
-                /// CDC-FLAG ///
                 $newpageid = $answer->jumpto;
                 if (!$response = trim($answer->response)) {
                     if ($correctanswer) {
@@ -357,10 +404,9 @@
                         $response = get_string("thatsthewronganswer", "lesson");
                     }
                 }
+                $studentanswer = $answer->answer;
             }
             break;
-        
-        /// CDC-FLAG /// 6/14/04  -- added responses    
         case LESSON_MATCHING :
             if (isset($_POST['response']) && is_array($_POST['response'])) { // only arrays should be submitted
                 $response = array();
@@ -399,24 +445,25 @@
                 $i++;
             }
             // get he users exact responses for record keeping
-            foreach ($response as $value) {
+            $userresponse = array();
+            foreach ($response as $key => $value) {
                 foreach($answers as $answer) {
                     if ($value == $answer->response) {
                         $userresponse[] = $answer->id;
                     }
                 }
+                $studentanswer .= '<br />'.$answers[$key]->answer.' = '.$value;
             }
             $userresponse = implode(",", $userresponse);
 
             if ($ncorrect == count($answers)-2) {  // dont count correct/wrong responses in the total.
-                   $response = get_string("thatsthecorrectanswer", "lesson");
+                $response = get_string("thatsthecorrectanswer", "lesson");
                 foreach ($answers as $answer) {
                     if ($answer->response == NULL && $answer->answer != NULL) {
                         $response = $answer->answer;
                         break;
                     }
                 }
-                // NoticeFix
                 if (isset($correctpageid)) {
                     $newpageid = $correctpageid;
                 }
@@ -425,7 +472,7 @@
                 }
                 $correctanswer = true;
             } else {
-                   $response = get_string("thatsthewronganswer", "lesson");
+                $response = get_string("thatsthewronganswer", "lesson");
                 $t = 0;
                 foreach ($answers as $answer) {
                     if ($answer->response == NULL && $answer->answer != NULL) {
@@ -440,7 +487,6 @@
                 $answerid = $wronganswerid;
             }
             break;
-            /// CDC-FLAG ///
 
         case LESSON_NUMERICAL :
             // set defaults
@@ -474,16 +520,14 @@
                     if (lesson_iscorrect($pageid, $newpageid)) {
                         $correctanswer = true;
                     }
-                    /// CDC-FLAG ///
                     if ($lesson->custom) {
                         if ($answer->score > 0) {
                             $correctanswer = true;
-                            $answerid = $answer->id;
                         } else {
                             $correctanswer = false;
                         }
                     }
-                    /// CDC-FLAG ///
+                    $answerid = $answer->id;
                     break;
                 }
             }
@@ -501,7 +545,7 @@
         case LESSON_BRANCHTABLE:
             $noanswer = false;
             $newpageid = optional_param('jumpto', NULL, PARAM_INT);
-            /// CDC-FLAG /// 6/15/04 going to insert into lesson_branch                
+            // going to insert into lesson_branch                
             if ($newpageid == LESSON_RANDOMBRANCH) {
                 $branchflag = 1;
             } else {
@@ -524,17 +568,15 @@
             if (!insert_record("lesson_branch", $branch)) {
                 error("Error: could not insert row into lesson_branch table");
             }
-            /// CDC-FLAG ///
 
-            /// CDC-FLAG ///  this is called when jumping to random from a branch table
+            //  this is called when jumping to random from a branch table
             if($newpageid == LESSON_UNSEENBRANCHPAGE) {
                 if (isteacher($course->id)) {
                      $newpageid = LESSON_NEXTPAGE;
                 } else {
-                     $newpageid = lesson_unseen_question_jump($lesson->id, $USER->id, $pageid);  // this may return 0 //CDC Chris Berri.....this is where it sets the next page id for unseen?
+                     $newpageid = lesson_unseen_question_jump($lesson->id, $USER->id, $pageid);  // this may return 0
                 }
             }
-            /// CDC-FLAG 6/15/04 ///
             // convert jumpto page into a proper page id
             if ($newpageid == 0) {
                 $newpageid = $pageid;
@@ -543,14 +585,13 @@
                     // no nextpage go to end of lesson
                     $newpageid = LESSON_EOL;
                 }
-    /* CDC-FLAG */  } elseif ($newpageid == LESSON_PREVIOUSPAGE) {
+            } elseif ($newpageid == LESSON_PREVIOUSPAGE) {
                 $newpageid = $page->prevpageid;
             } elseif ($newpageid == LESSON_RANDOMPAGE) {
                 $newpageid = lesson_random_question_jump($lesson->id, $pageid);
-            } elseif ($newpageid == LESSON_RANDOMBRANCH) {  // 6/15/04
+            } elseif ($newpageid == LESSON_RANDOMBRANCH) {
                 $newpageid = lesson_unseen_branch_jump($lesson->id, $USER->id);
             }
-            /// CDC-FLAG ///
             // no need to record anything in lesson_attempts            
             redirect("view.php?id=$cm->id&amp;action=navigation&amp;pageid=$newpageid");
             print_footer($course);
@@ -558,6 +599,10 @@
             break;
         
     }
+
+    $attemptsremaining = 0;
+    $maxattemptsreached = 0;
+
     if ($noanswer) {
         $newpageid = $pageid; // display same page again
         print_simple_box(get_string("noanswer", "lesson"), "center");
@@ -575,8 +620,9 @@
             if(isset($userresponse)) {
                 $attempt->useranswer = $userresponse;
             }
+            
             $attempt->timeseen = time();
-            /// CDC-FLAG /// -- dont want to insert the attempt if they ran out of time
+            // dont want to insert the attempt if they ran out of time
             if (!$outoftime) {
                 // if allow modattempts, then update the old attempt record, otherwise, insert new answer record
                 if (isset($USER->modattempts[$lesson->id])) {
@@ -586,20 +632,22 @@
                     error("Continue: attempt not inserted");
                 }
             }
-            /// CDC-FLAG ///
+            // "number of attempts remaining" message if $lesson->maxattempts > 1
+            // displaying of message(s) is at the end of page for more ergonomic display
             if (!$correctanswer and ($newpageid == 0)) {
                 // wrong answer and student is stuck on this page - check how many attempts 
                 // the student has had at this page/question
                 $nattempts = count_records("lesson_attempts", "pageid", $pageid, "userid", $USER->id,
                     "retry", $nretakes);
-
+                
+                // retreive the number of attempts left counter for displaying at bottom of feedback page
                 if ($nattempts >= $lesson->maxattempts) {
                     if ($lesson->maxattempts > 1) { // don't bother with message if only one attempt
-                        echo "<p align=\"center\">(".get_string("maximumnumberofattempts", "lesson").
-                            " ".get_string("reached", "lesson")." - ".
-                            get_string("movingtonextpage", "lesson").")</p>\n";
+                        $maxattemptsreached = 1;
                     }
                     $newpageid = LESSON_NEXTPAGE;
+                } else if ($lesson->maxattempts > 1) { // don't bother with message if only one attempt
+                    $attemptsremaining = $lesson->maxattempts - $nattempts;
                 }
             }
         }
@@ -656,43 +704,49 @@
             }
         }
     
-        /// CDC-FLAG 6/21/04 ///  this calculates the ongoing score
+        // This calculates and prints the ongoing score message
         if ($lesson->ongoing) {
-            if (isteacher($course->id)) {
-                echo "<div align=\"center\">".get_string("teacherongoingwarning", "lesson")."</div><br>";
-            } else {
-                $ntries = count_records("lesson_grades", "lessonid", $lesson->id, "userid", $USER->id);
-                if (isset($USER->modattempts[$lesson->id])) {
-                    $ntries--;
-                }
-                lesson_calculate_ongoing_score($lesson, $USER->id, $ntries);
-            }
+            lesson_print_ongoing_score($lesson);
         }
-        /// CDC-FLAG ///
 
         // display response (if there is one - there should be!)
+        // display: lesson title, page title, question text, student's answer(s) before feedback message
         if ($response) {
-            //$title = get_field("lesson_pages", "title", "id", $pageid);
-            //print_heading($title);
+            //optionally display question page title
+            //if ($title = get_field("lesson_pages", "title", "id", $pageid)) {
+            //    print_heading($title);
+            //}
             echo "<table width=\"80%\" border=\"0\" align=\"center\"><tr><td>\n";
             if ($lesson->review && !$correctanswer && !$isessayquestion) {
                 $nretakes = count_records("lesson_grades", "lessonid", $lesson->id, "userid", $USER->id); 
                 $qattempts = count_records("lesson_attempts", "userid", $USER->id, "retry", $nretakes, "pageid", $pageid);
-                echo "<br><br>";
+                echo "<br /><br />";
                 if ($qattempts == 1) {
                     print_simple_box(get_string("firstwrong", "lesson"), "center");
                 } else {
                     print_simple_box(get_string("secondpluswrong", "lesson"), "center");
                 }
             } else {
-                print_simple_box(format_text($response), 'center');
+                if ($correctanswer) {
+                    $class = 'response correct'; //CSS over-ride this if they exist (!important)
+                } else if ($isessayquestion) {
+                    $class = 'response';
+                } else {
+                    $class = 'response incorrect'; 
+                }
+                $options = new stdClass;
+                $options->noclean = true;
+                $options->para = false;
+                print_simple_box(format_text($page->contents, FORMAT_MOODLE, $options), 'center');
+                echo '<br />';
+                print_simple_box('<em>'.get_string("youranswer", "lesson").'</em> : '.format_text($studentanswer, FORMAT_MOODLE, $options).
+                                 "<div class=\"$class\">".format_text($response, FORMAT_MOODLE, $options), 'center').'</div>';
             }
             echo "</td></tr></table>\n";
         }
     }
 
-
-    /// CDC-FLAG 6/18/04 ///  - this is where some jump numbers are interpreted
+    // this is where some jump numbers are interpreted
     if($outoftime) {
         $newpageid = LESSON_EOL;  // ran out of time for the test, so go to eol
     } elseif (isset($USER->modattempts[$lesson->id])) {
@@ -759,6 +813,7 @@
         }
     }
 
+    // NOTE:  Should this code be coverted from form/javascript to just longer links with the variables in the url?
     echo "<form name=\"pageform\" method =\"post\" action=\"view.php\">\n";
     echo "<input type=\"hidden\" name=\"id\" value=\"$cm->id\">\n";
     echo "<input type=\"hidden\" name=\"action\" value=\"navigation\">\n";
@@ -767,22 +822,31 @@
     if (isset($USER->modattempts[$lesson->id])) {
         echo "<p align=\"center\">".
             get_string("savechangesandeol", "lesson")."<br /><br />".
-            "<input type=\"submit\" onClick='pageform.pageid.value=".LESSON_EOL.";' name=\"save\" value=\"".
-            get_string("savechanges", "lesson")."\"></p>\n";
+            "<div align=\"center\" class=\"lessonbutton standardbutton\"><a href=\"javascript:document.pageform.pageid.value=".LESSON_EOL.";document.pageform.submit();\">".
+            get_string("savechanges", "lesson")."</a></div></p>\n";
         echo "<p align=\"center\">".get_string("or", "lesson")."<br /><br />".
             get_string("continuetoanswer", "lesson")."</p>\n";
     }
 
     if ($lesson->review && !$correctanswer && !$noanswer && !$isessayquestion) {
-        echo "<p align=\"center\"><input type=\"submit\" onClick='pageform.pageid.value=$pageid;' name=\"review\" value=\"".
-            get_string("reviewquestionback", "lesson")."\"></p>\n";
-        echo "<p align=\"center\"><input type=\"submit\" name=\"continue\" value=\"".
-            get_string("reviewquestioncontinue", "lesson")."\"></p>\n";
+        echo "<p><div align=\"center\" class=\"lessonbutton standardbutton\"><a href=\"javascript:document.pageform.pageid.value=$pageid;document.pageform.submit();\">".
+            get_string("reviewquestionback", "lesson")."</a></div></p>\n";
+        echo "<p><div align=\"center\" class=\"lessonbutton standardbutton\"><a href=\"javascript:document.pageform.submit();\">".
+            get_string("reviewquestioncontinue", "lesson")."</a></div></p>\n";
     } else {
-        echo "<p align=\"center\"><input type=\"submit\" name=\"continue\" value=\"".
-            get_string("continue", "lesson")."\"></p>\n";
+        echo "<p><div align=\"center\" class=\"lessonbutton standardbutton\"><a href=\"javascript:document.pageform.submit();\">".
+            get_string("continue", "lesson")."</a></div></p>\n";
     }
     echo "</form>\n";
+    
+/// Report attempts remaining
+    if ($attemptsremaining != 0) {
+        echo "<p align=\"center\">".get_string('attemptsremaining', 'lesson', $attemptsremaining);
+    }
+/// Report if max attempts reached
+    if ($maxattemptsreached != 0) {
+        echo "<p align=\"center\">(".get_string("maximumnumberofattemptsreached", "lesson").")</p>\n";
+    }
 
     if ($lesson->displayleft) {
         echo "</td></tr></table>";
