@@ -1,10 +1,10 @@
-<?php // $Id: mysql.php,v 1.68.2.2 2005/06/21 07:05:51 gustav_delius Exp $
+<?php // $Id: mysql.php,v 1.100.2.2 2006/06/07 05:57:00 toyomoyo Exp $
 
 function quiz_upgrade($oldversion) {
 // This function does anything necessary to upgrade
 // older versions to match current functionality
 
-    global $CFG, $QUIZ_QTYPES, $db;
+    global $CFG, $QTYPES, $db;
     require_once("$CFG->dirroot/mod/quiz/locallib.php");
 
     if ($oldversion < 2002101800) {
@@ -16,10 +16,10 @@ function quiz_upgrade($oldversion) {
     }
     if ($oldversion < 2002102101) {
         execute_sql(" DELETE FROM log_display WHERE module = 'quiz' ");
-        execute_sql(" INSERT INTO log_display VALUES ('quiz', 'view', 'quiz', 'name') ");
-        execute_sql(" INSERT INTO log_display VALUES ('quiz', 'report', 'quiz', 'name') ");
-        execute_sql(" INSERT INTO log_display VALUES ('quiz', 'attempt', 'quiz', 'name') ");
-        execute_sql(" INSERT INTO log_display VALUES ('quiz', 'submit', 'quiz', 'name') ");
+        execute_sql(" INSERT INTO log_display (module, action, mtable, field) VALUES ('quiz', 'view', 'quiz', 'name') ");
+        execute_sql(" INSERT INTO log_display (module, action, mtable, field) VALUES ('quiz', 'report', 'quiz', 'name') ");
+        execute_sql(" INSERT INTO log_display (module, action, mtable, field) VALUES ('quiz', 'attempt', 'quiz', 'name') ");
+        execute_sql(" INSERT INTO log_display (module, action, mtable, field) VALUES ('quiz', 'submit', 'quiz', 'name') ");
     }
     if ($oldversion < 2002102600) {
         execute_sql(" ALTER TABLE `quiz_answers` CHANGE `feedback` `feedback` TEXT NOT NULL ");
@@ -107,7 +107,7 @@ function quiz_upgrade($oldversion) {
     }
 
     if ($oldversion < 2003072400) {
-        execute_sql(" INSERT INTO {$CFG->prefix}log_display VALUES ('quiz', 'review', 'quiz', 'name') ");
+        execute_sql(" INSERT INTO {$CFG->prefix}log_display (module, action, mtable, field) VALUES ('quiz', 'review', 'quiz', 'name') ");
     }
 
     if ($oldversion < 2003072901) {
@@ -187,8 +187,8 @@ function quiz_upgrade($oldversion) {
     }
 
     if ($oldversion < 2004021900) {
-        modify_database("","INSERT INTO prefix_log_display VALUES ('quiz', 'add', 'quiz', 'name');");
-        modify_database("","INSERT INTO prefix_log_display VALUES ('quiz', 'update', 'quiz', 'name');");
+        modify_database("","INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('quiz', 'add', 'quiz', 'name');");
+        modify_database("","INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('quiz', 'update', 'quiz', 'name');");
     }
 
     if ($oldversion < 2004051700) {
@@ -328,7 +328,7 @@ function quiz_upgrade($oldversion) {
     }
 
     if ($oldversion < 2005032000) {
-        execute_sql(" INSERT INTO {$CFG->prefix}log_display VALUES ('quiz', 'editquestions', 'quiz', 'name') ");
+        execute_sql(" INSERT INTO {$CFG->prefix}log_display (module, action, mtable, field) VALUES ('quiz', 'editquestions', 'quiz', 'name') ");
     }
 
     if ($oldversion < 2005032300) {
@@ -479,9 +479,9 @@ function quiz_upgrade($oldversion) {
 
     /// Entries for the log_display table
 
-        modify_database('', " INSERT INTO prefix_log_display VALUES ('quiz', 'preview', 'quiz', 'name');");
-        modify_database('', " INSERT INTO prefix_log_display VALUES ('quiz', 'start attempt', 'quiz', 'name');");
-        modify_database('', " INSERT INTO prefix_log_display VALUES ('quiz', 'close attempt', 'quiz', 'name');");
+        modify_database('', " INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('quiz', 'preview', 'quiz', 'name');");
+        modify_database('', " INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('quiz', 'start attempt', 'quiz', 'name');");
+        modify_database('', " INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('quiz', 'close attempt', 'quiz', 'name');");
 
     /// update the default settings in $CFG
         $review = (QUIZ_REVIEW_IMMEDIATELY & (QUIZ_REVIEW_RESPONSES + QUIZ_REVIEW_SCORES));
@@ -666,6 +666,7 @@ function quiz_upgrade($oldversion) {
                     $multianswers[$i]->length = 0;
                     $multianswers[$i]->questiontext = '';
                     $multianswers[$i]->stamp = make_unique_id_code();
+                    $multianswers[$i]->name = addslashes($multianswers[$i]->name);
                     // $multianswers[$i]->parent is set in the query
                     // $multianswers[$i]->defaultgrade is set in the query
                     // $multianswers[$i]->qtype is set in the query
@@ -839,6 +840,224 @@ function quiz_upgrade($oldversion) {
         $db->debug = $olddebug;
     }
 
+    if ($oldversion < 2005062600) {
+        modify_database ('', "
+            CREATE TABLE `prefix_quiz_essay` (
+                `id` int(10) unsigned NOT NULL auto_increment,
+                `question` int(10) unsigned NOT NULL default '0',
+                `answer` varchar(255) NOT NULL default '',
+                PRIMARY KEY  (`id`),
+                KEY `question` (`question`)
+           ) TYPE=MyISAM COMMENT='Options for essay questions'");
+    
+        modify_database ('', "
+            CREATE TABLE `prefix_quiz_essay_states` (
+              `id` int(10) unsigned NOT NULL auto_increment,
+              `stateid` int(10) unsigned NOT NULL default '0',
+              `graded` tinyint(4) unsigned NOT NULL default '0',
+              `fraction` varchar(10) NOT NULL default '0.0',
+              `response` text NOT NULL,
+              PRIMARY KEY  (`id`)
+            ) TYPE=MyISAM COMMENT='essay question type specific state information'");
+    }
+
+    if ($oldversion < 2005070202) {
+        // add new unique id to prepare the way for lesson module to have its own attempts table
+        table_column('quiz_attempts', '', 'uniqueid', 'integer', '10', 'unsigned', '0', 'not null', 'id');
+        // initially we can use the id as the unique id because no other modules use attempts yet.
+        execute_sql("UPDATE {$CFG->prefix}quiz_attempts SET uniqueid = id", false);
+        // we set $CFG->attemptuniqueid to the next available id
+        $record = get_record_sql("SELECT max(id)+1 AS nextid FROM {$CFG->prefix}quiz_attempts");
+        set_config('attemptuniqueid', empty($record->nextid) ? 1 : $record->nextid);
+    }
+    
+    if ($oldversion < 2006020801) {
+        // add new field to store time delay between the first and second quiz attempt
+        table_column('quiz', '', 'delay1', 'integer', '10', 'unsigned', '0', 'not null', 'popup');
+        // add new field to store time delay between the second and any additional quizes
+        table_column('quiz', '', 'delay2', 'integer', '10', 'unsigned', '0', 'not null', 'delay1');
+    }
+
+    if ($oldversion < 2006021101) {
+        // set defaultgrade field properly (probably not necessary, but better make sure)
+        execute_sql("UPDATE {$CFG->prefix}quiz_questions SET defaultgrade = '1' WHERE defaultgrade = '0'", false);
+        execute_sql("UPDATE {$CFG->prefix}quiz_questions SET defaultgrade = '0' WHERE qtype = '".DESCRIPTION."'", false);
+    }
+
+    if ($oldversion < 2006021103) {
+        // add new field to store the question-level shuffleanswers option
+        table_column('quiz_match', '', 'shuffleanswers', 'tinyint', '4', 'unsigned', '1', 'not null', 'subquestions');
+        table_column('quiz_multichoice', '', 'shuffleanswers', 'tinyint', '4', 'unsigned', '1', 'not null', 'single');
+        table_column('quiz_randomsamatch', '', 'shuffleanswers', 'tinyint', '4', 'unsigned', '1', 'not null', 'choose');
+    }
+
+    if ($oldversion < 2006021104) {
+        // add originalversion field for the new versioning mechanism
+        table_column('quiz_question_versions', '', 'originalquestion', 'int', '10', 'unsigned', '0', 'not null', 'newquestion');
+    }
+
+    if ($oldversion < 2006021301) {
+        modify_database('','ALTER TABLE prefix_quiz_attempts ADD UNIQUE INDEX uniqueid (uniqueid);');
+    }
+
+    if ($oldversion < 2006021302) {
+        table_column('quiz_match_sub', '', 'code', 'int', '10', 'unsigned', '0', 'not null', 'id');
+        execute_sql("UPDATE {$CFG->prefix}quiz_match_sub SET code = id", false);
+    }
+    if ($oldversion < 2006021304) {
+        // convert sequence field to text to accomodate very long sequences, see bug 4257
+        table_column('quiz_multianswers', 'sequence', 'sequence',  'text', '', '', '', 'not null', 'question');
+    }
+
+    if ($oldversion < 2006021501) {
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_newest_states RENAME {$CFG->prefix}question_sessions", false);
+    }
+
+    if ($oldversion < 2006022200) {
+        // convert grade fields to float
+        set_field('quiz_attempts', 'sumgrades', 0, 'sumgrades', '');
+        table_column('quiz_attempts', 'sumgrades', 'sumgrades',  'float', '', '', '0', 'not null');
+
+        set_field('quiz_answers', 'fraction', 0, 'fraction', '');
+        table_column('quiz_answers', 'fraction', 'fraction',  'float', '', '', '0', 'not null');
+
+        set_field('quiz_essay_states', 'fraction', 0, 'fraction', '');
+        table_column('quiz_essay_states', 'fraction', 'fraction',  'float', '', '', '0', 'not null');
+
+        set_field('quiz_states', 'grade', 0, 'grade', '');
+        table_column('quiz_states', 'grade', 'grade',  'float', '', '', '0', 'not null');
+
+        set_field('quiz_states', 'raw_grade', 0, 'raw_grade', '');
+        table_column('quiz_states', 'raw_grade', 'raw_grade',  'float', '', '', '0', 'not null');
+
+        set_field('quiz_states', 'penalty', 0, 'penalty', '');
+        table_column('quiz_states', 'penalty', 'penalty',  'float', '', '', '0', 'not null');
+
+        set_field('question_sessions', 'sumpenalty', 0, 'sumpenalty', '');
+        table_column('question_sessions', 'sumpenalty', 'sumpenalty',  'float', '', '', '0', 'not null');
+    }
+
+    if ($oldversion < 2006022400) {
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_questions RENAME {$CFG->prefix}question", false);
+    }
+
+    if ($oldversion < 2006022402) {
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_states RENAME {$CFG->prefix}question_states", false);
+    }
+
+    if ($oldversion < 2006022800) {
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_answers RENAME {$CFG->prefix}question_answers", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_categories RENAME {$CFG->prefix}question_categories", false);
+    }
+
+    if ($oldversion < 2006031202) {
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_truefalse RENAME {$CFG->prefix}question_truefalse", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_shortanswer RENAME {$CFG->prefix}question_shortanswer", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_multianswers RENAME {$CFG->prefix}question_multianswer", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_multichoice RENAME {$CFG->prefix}question_multichoice", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_numerical RENAME {$CFG->prefix}question_numerical", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_numerical_units RENAME {$CFG->prefix}question_numerical_units", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_randomsamatch RENAME {$CFG->prefix}question_randomsamatch", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_match RENAME {$CFG->prefix}question_match", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_match_sub RENAME {$CFG->prefix}question_match_sub", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_calculated RENAME {$CFG->prefix}question_calculated", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_dataset_definitions RENAME {$CFG->prefix}question_dataset_definitions", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_dataset_items RENAME {$CFG->prefix}question_dataset_items", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_question_datasets RENAME {$CFG->prefix}question_datasets", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_rqp RENAME {$CFG->prefix}question_rqp", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_rqp_servers RENAME {$CFG->prefix}question_rqp_servers", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_rqp_states RENAME {$CFG->prefix}question_rqp_states", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_rqp_types RENAME {$CFG->prefix}question_rqp_types", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_essay RENAME {$CFG->prefix}question_essay", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}quiz_essay_states RENAME {$CFG->prefix}question_essay_states", false);
+    }
+
+    if ($oldversion < 2006032100) {
+        // change from the old questiontype numbers to using the questiontype names
+        table_column('question', 'qtype', 'qtype',  'varchar', 20, '', '', 'not null');
+        set_field('question', 'qtype', 'shortanswer', 'qtype', 1);
+        set_field('question', 'qtype', 'truefalse', 'qtype', 2);
+        set_field('question', 'qtype', 'multichoice', 'qtype', 3);
+        set_field('question', 'qtype', 'random', 'qtype', 4);
+        set_field('question', 'qtype', 'match', 'qtype', 5);
+        set_field('question', 'qtype', 'randomsamatch', 'qtype', 6);
+        set_field('question', 'qtype', 'description', 'qtype', 7);
+        set_field('question', 'qtype', 'numerical', 'qtype', 8);
+        set_field('question', 'qtype', 'multianswer', 'qtype', 9);
+        set_field('question', 'qtype', 'calculated', 'qtype', 10);
+        set_field('question', 'qtype', 'rqp', 'qtype', 11);
+        set_field('question', 'qtype', 'essay', 'qtype', 12);
+    }
+
+    if ($oldversion < 2006032200) {
+        // set version for all questiontypes that already have their tables installed
+        set_config('qtype_calculated_version', 2006032100);
+        set_config('qtype_essay_version', 2006032100);
+        set_config('qtype_match_version', 2006032100);
+        set_config('qtype_multianswer_version', 2006032100);
+        set_config('qtype_multichoice_version', 2006032100);
+        set_config('qtype_numerical_version', 2006032100);
+        set_config('qtype_randomsamatch_version', 2006032100);
+        set_config('qtype_rqp_version', 2006032100);
+        set_config('qtype_shortanswer_version', 2006032100);
+        set_config('qtype_truefalse_version', 2006032100);
+    }
+
+    if ($oldversion < 2006040600) {
+        table_column('question_sessions', '', 'comment', 'text', '', '', '', 'not null', 'sumpenalty');
+    }
+
+    if ($oldversion < 2006040900) {
+        modify_database('', "UPDATE prefix_question SET parent = id WHERE qtype ='random';");
+    }
+
+    if ($oldversion < 2006041000) {
+        modify_database('', " INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('quiz', 'continue attempt', 'quiz', 'name');");
+    }
+
+    if ($oldversion < 2006041001) {
+        table_column('question', 'version', 'version', 'varchar', 255);
+    }
+
+    if ($oldversion < 2006042800) {
+        // Check we have some un-renamed tables (verified in some servers)
+        if ($tables = $db->MetaTables('TABLES')) {
+            if (in_array($CFG->prefix.'quiz_randommatch', $tables) &&
+                !in_array($CFG->prefix.'question_randomsamatch', $tables)) {
+                modify_database ("", "ALTER TABLE prefix_quiz_randommatch RENAME prefix_question_randomsamatch ");
+            }
+            // Check for one possible missing field in one table
+            if ($columns = $db->MetaColumnNames($CFG->prefix.'question_randomsamatch')) {
+                if (!in_array('shuffleanswers', $columns)) {
+                    table_column('question_randomsamatch', '', 'shuffleanswers', 'tinyint', '4', 'unsigned', '1', 'not null', 'choose');
+                }
+            }
+        }
+    }
+
+    if ($oldversion < 2006042801) {
+        // The newgraded field must always point to a valid state
+        modify_database("","UPDATE prefix_question_sessions SET newgraded = newest where newgraded = '0'");
+
+        // The following table is discussed in bug 5468
+        modify_database ("", "CREATE TABLE prefix_question_attempts (
+                                  id int(10) unsigned NOT NULL auto_increment,
+                                  modulename varchar(20) NOT NULL default 'quiz',
+                                  PRIMARY KEY  (id)
+                                ) TYPE=MyISAM COMMENT='Student attempts. This table gets extended by the modules';");
+        // create one entry for all the existing quiz attempts
+        modify_database ("", "INSERT INTO prefix_question_attempts (id)
+                                   SELECT uniqueid
+                                   FROM prefix_quiz_attempts;");
+    }
+
+    if ($oldversion < 2006060700) { // fix for 5720
+        execute_sql('DROP TABLE '.$CFG->prefix.'question_essay_states', false);
+        execute_sql('DROP TABLE '.$CFG->prefix.'question_essay', false);
+        execute_sql('DROP TABLE '.$CFG->prefix.'quiz_attemptonlast_datasets', false);
+    }
+
+
     return true;
 }
 
